package com.sample;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import junit.framework.TestCase;

import org.jbpm.runtime.manager.impl.RuntimeEnvironmentBuilder;
import org.jbpm.services.task.identity.JBossUserGroupCallbackImpl;
import org.junit.Test;
import org.kie.api.io.ResourceType;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.manager.RuntimeEngine;
import org.kie.api.runtime.manager.RuntimeEnvironment;
import org.kie.api.runtime.manager.RuntimeManager;
import org.kie.api.runtime.manager.RuntimeManagerFactory;
import org.kie.api.runtime.process.ProcessInstance;
import org.kie.api.task.TaskService;
import org.kie.api.task.UserGroupCallback;
import org.kie.api.task.model.Task;
import org.kie.internal.io.ResourceFactory;
import org.kie.internal.runtime.manager.context.EmptyContext;

import bitronix.tm.BitronixTransactionManager;
import bitronix.tm.TransactionManagerServices;
import bitronix.tm.resource.jdbc.PoolingDataSource;

/**
 * This is a sample file to launch a process.
 */
public class ProcessMainJPATest extends TestCase {

    private static EntityManagerFactory emf;

    @Test
    public void testParallelTaskCancel() throws Exception {
        
        try {
            
            setup();

            for (int i = 0; i < 5; i++) {
                RuntimeManager manager = getRuntimeManager("sample.bpmn");
                RuntimeEngine runtime = manager.getRuntimeEngine(EmptyContext.get());
                KieSession ksession = runtime.getKieSession();

                Map<String, Object> params = new HashMap<String, Object>();
                ProcessInstance pi = ksession.startProcess("com.sample.bpmn.hello", params);
                System.out.println("A process instance started : pid = " + pi.getId());
                
                long piid = pi.getId();
                TaskService taskService = runtime.getTaskService();
                // -----------
                {
                    List<Long> list = taskService.getTasksByProcessInstanceId(piid);
                    for (long taskId : list) {
                        Task task = taskService.getTaskById(taskId);
                        System.out.println("taskId = " + task.getId() + ", status = " + task.getTaskData().getStatus());
                    }
                }
                
                manager.disposeRuntimeEngine(runtime);
                manager.close();
            }

        // -----------
        
        
        } catch (Throwable th) {
            th.printStackTrace();
        }

        System.exit(0);
    }
    
    private static void setup() {
        // for H2 datasource
//        JBPMHelper.startH2Server();
//        JBPMHelper.setupDataSource();

        // for external database datasource
         setupDataSource();
        
        Map configOverrides = new HashMap();
        configOverrides.put("hibernate.hbm2ddl.auto", "create"); // Uncomment if you don't want to clean up tables
//        configOverrides.put("hibernate.dialect", "org.hibernate.dialect.H2Dialect"); // Edit for other databases
//        configOverrides.put("hibernate.dialect", "org.hibernate.dialect.MySQL5InnoDBDialect");
        configOverrides.put("hibernate.dialect", "org.hibernate.dialect.Oracle10gDialect");

        
        emf = Persistence.createEntityManagerFactory("org.jbpm.persistence.jpa", configOverrides);
    }

    private static RuntimeManager getRuntimeManager(String process) {


        Properties properties = new Properties();
        properties.setProperty("krisv", "");
        properties.setProperty("mary", "");
        properties.setProperty("john", "");
        UserGroupCallback userGroupCallback = new JBossUserGroupCallbackImpl(properties);


        RuntimeEnvironment environment =
                RuntimeEnvironmentBuilder.getDefault()
                .persistence(true)
                .entityManagerFactory(emf)
                .userGroupCallback(userGroupCallback)
                .addAsset(ResourceFactory.newClassPathResource(process), ResourceType.BPMN2)
                .get();
        return RuntimeManagerFactory.Factory.get().newPerProcessInstanceRuntimeManager(environment);

    }

    public static PoolingDataSource setupDataSource() {
        // Please edit here when you want to use your database
        PoolingDataSource pds = new PoolingDataSource();
        pds.setUniqueName("jdbc/jbpm-ds");
        pds.setClassName("bitronix.tm.resource.jdbc.lrc.LrcXADataSource");
        pds.setMaxPoolSize(5);
        pds.setAllowLocalTransactions(true);
        pds.getDriverProperties().put("user", "jboss");
        pds.getDriverProperties().put("password", "jboss");
        pds.getDriverProperties().put("url", "jdbc:oracle:thin:@lab-jboss-db.gsslab.pnq.redhat.com:1521:orcl");
        pds.getDriverProperties().put("driverClassName", "oracle.jdbc.OracleDriver");
        pds.init();
        return pds;
    }
}